#!/usr/bin/env python
# −*− coding: UTF−8 −*−
import os, sys, ConfigParser, getpass
import requests

def read_config():
    config_file = 'rbc.cfg'
    for loc in os.curdir, "%s/.rbc" % os.path.expanduser("~"), "/etc", os.environ.get('PWD'):
        if os.path.exists(os.path.join(loc,config_file)):
            try:
                config = ConfigParser.ConfigParser()
                config.read(os.path.join(loc,config_file))
                return config
            except IOError:
                pass
    if (os.environ.get("RBC_SECRETKEY") and os.environ.get("RBC_APIKEY")):
        # Initialize using ENV
        return ConfigParser.ConfigParser()
    else:
        # Present an option to create a config file
        print "Could not find a config file (%s) in ~/rbc/, /etc or current dir." % config_file
        print "You may create one by running rbc-setup"
        sys.exit(1)

def main():
    user = get_keys_from_login()
    config = ConfigParser.RawConfigParser()
    config.add_section('main')
    config.set('main', 'apikey', user['user']['apikey'])
    config.set('main', 'secretkey', user['user']['secretkey'])
    config.set('main', 'endpoint', 'api.rbcloud.net')
    d = os.path.join(os.path.expanduser('~'), '.rbc')
    if not os.path.exists(d):
        os.makedirs(d)
    with open(os.path.join(d,'rbc.cfg'), 'wb') as configfile:
        config.write(configfile)

def get_keys_from_login():
    user = raw_input('Enter your rbc username:')
    password = getpass.getpass('Enter your rbc password:')
    url = "https://api.rbcloud.net/client/api"
    s = requests.Session()
    r = s.post(url, data={'username': user, 'password': password, 'domain': '/', 'command': 'login', 'response': 'json'})
    if r.status_code == 200:
        # login ok
        resp = requests.get(url, params={'command': 'updateUser', 
                'id':r.json()['loginresponse']['userid'], 'response':'json', 
                'sessionkey': r.json()['loginresponse']['sessionkey']},
                cookies=dict(JSESSIONID=r.cookies['JSESSIONID'], Path="/")
        )
        if resp.status_code == 200:
            return resp.json()['updateuserresponse']
        else:
            print "Unable to login, please verify your login and password"
            sys.exit(1)
    else:
        print "Unable to perform initial login"
        sys.exit(1)






#!/usr/bin/env python
# −*− coding: UTF−8 −*−
"""Manage port forwarding rules in RedBridge Cloud

Usage:
  rbc-portforward list [-n] [--short] [--network NETWORK] [--pubip IPADDRESS]
  rbc-portforward add INSTANCE (--pubip IPADDRESS|--network NETWORK) --forward RULE...
  rbc-portforward delete UUID

Options:
  -n --noheader     Do not show the table header
  --short           Short display, only shows name
  --network=NETWORK A network name, default public IP for the specified net is used 
  --pubip=IPADDRESS A public IP address
  --forward=RULE    A forward rule in the format proto/publicport:privateport (tcp/22:22)
  -h --help         Show this screen.
  --version         Show version.

"""
import sys, operator, os, random, string
from tabulate import tabulate
from config import read_config
from connection import conn
from zone import get_zone
from docopt import docopt

__cli_name__="rbc-portforward"
from rbc_tools import __version__

def get_default_snat_id(c, id):
    ip = c.list_publicipaddresses(associatednetworkid=id, issourcenat="true")
    if ip:
        return ip[0].id
    else:
        return ['N/A']

def get_network_from_name(c, name):
    ret = c.list_networks(keyword=name)
    if len(ret) == 1:
        return ret[0]
    else:
        print "Unable to get network, got: %s" % ret
        sys.exit(1)


def get_instance_from_name(c, name, networkid=None):
    if networkid:
        ret = c.list_virtualmachines(keyword=name, name=name, networkid=networkid)
    else:
        ret = c.list_virtualmachines(keyword=name, name=name)
    if len(ret) == 1:
        return ret[0]
    else:
        print "Unable to get instance, got: %s" % ret
        sys.exit(1)

def get_id_from_address(c, address):
    try:
        res = c.list_publicipaddresses(ipaddress=address)[0]
        if res:
            return res
        else:
            print "Unable to get ip address id"
            sys.exit(0)
    except:
        print "Unable to get ip address id"
        sys.exit(1)

def print_tabulate(res, noheader=False, short=False, t='list'):
    config = read_config()
    c = conn(config)
    tbl = []
    for i in res:
        if t == 'list':
            if not i.__dict__.has_key('virtualmachinename'):
                i.__setattr__('virtualmachinename', 'None')
            if short:
                tbl.append([i.id])
            else:
                tbl.append([
                    i.ipaddress,
                    c.list_networks(id=i.networkid)[0].name,
                    i.publicport,
                    i.privateport,
                    i.protocol,
                    i.virtualmachinename,
                    i.state,
                    i.id,
                ])

    tbl = sorted(tbl, key=operator.itemgetter(0))
    if (noheader or short):
        print tabulate(tbl, tablefmt="plain")
    else:
        tbl.insert(0, ['ip', 'network', 'public port', 'private port', 'proto',
            'vmname', 'state', 'uuid'])
        print tabulate(tbl, headers="firstrow")

def main():
    config = read_config()
    c = conn(config)
    args = docopt(__doc__, version='%s %s' % (__cli_name__, __version__))
    if args['list']:
        if args['--network']:
            res = c.list_portforwardingrules(networkid=get_network_from_name(c,
                args['--network']).id)
        elif args['--pubip']:
            res = c.list_portforwardingrules(ipaddressid=get_id_from_address(c,
                args['--pubip']).id)
        else:
            res = c.list_portforwardingrules()
    elif args['add']:
        try:
            if args['--pubip']:
                ipid = get_id_from_address(c, args['--pubip']).id
                network_id = get_id_from_address(c, args['--pubip']).networkid
            elif args['--network']:
                ipid = get_default_snat_id(c, get_network_from_name(c,
                    args['--network']).id)
                network_id = get_network_from_name(c, args['--network']).id
            else:
                print "Either --network or --pubip must be specified"
                sys.exit(1)
        except Exception as e:
            print "Unable to determine network or ip: %s" % e
            sys.exit(1)

        try:
            vmid = get_instance_from_name(c, args['INSTANCE'], networkid=network_id).id
        except Exception as e:
            print "Unable to find specified instance: %s" % e
            sys.exit(1)

        try:
            for f in args['--forward']:
                proto_port = f.split('/')
                ports=proto_port[1].split(':')
                async = c.create_portforwardingrule(
                    ipaddressid=ipid,
                    virtualmachineid=vmid,
                    publicport=ports[0],
                    privateport=ports[1],
                    protocol=proto_port[0])
                res = [async.get_result()]
                if not res:
                    raise Exception("An internal error occured.")
        except Exception as e:
            try:
                msg = e[1]
            except:
                msg = e
            print "Unable to create portforward rule: %s" % msg
            sys.exit(1)
    elif args['delete']:
        try:
            async = c.delete_portforwardingrule(id=args['UUID'])
            res = async.get_result()
            if not res:
                raise Exception("An internal error occured.")
            if res == 'succeded':
                sys.exit(0)
            else:
                raise Exception("Failure")
        except Exception as e:
            try:
                msg = e[1]
            except:
                msg = e
            print "Unable to delete portforward rule: %s" % msg
            sys.exit(1)

    if res:
        print_tabulate(res, noheader=args['--noheader'], short=args['--short'])
    else:
        print "Unable to execute command"
        sys.exit(1)



#!/usr/bin/env python
# −*− coding: UTF−8 −*−
"""Manage egress rules in RedBridge Cloud
Currently this tool only opens for all egress traffic

Usage:
  rbc-networks create [-n] [--short] NETWORK

Arguments:
  NETWORK network name

Options:
  -n --noheader                      Do not show the table header
  --short                            Short disply, only shows name
  -h --help      Show this screen.
  --version      Show version.

"""
import sys, operator
from tabulate import tabulate
from config import read_config
from connection import conn
from zone import get_zone
from docopt import docopt

__cli_name__="rbc-egress"
from rbc_tools import __version__

def get_network(c, name):
    nlist = []
    for n in c.list_networks(canusefordeploy="true"):
        if n.name == name:
            nlist.append(n)
    if nlist:
        return nlist
    else:
        print "The search returned no networks"
        sys.exit(0)

def main():
    config = read_config()
    c = conn(config)
    args = docopt(__doc__, version='%s %s' % (__cli_name__, __version__))
    if args['create']:
        id = get_network(c, args['NETWORK'])[0].id
        res = c.create_egressfirewallrule(networkid=id, protocol="All", cidrlist="0.0.0.0/0")
        if res:
            print "egress rule created"
            sys.exit(0)



#!/usr/bin/env python
# −*− coding: UTF−8 −*−
"""Manage public ip addresses in RedBridge Cloud

Usage:
  rbc-publicips list [-n] [--short] [NETWORK]
  rbc-publicips allocate NETWORK
  rbc-publicips release IPADDRESS

Options:
  -n --noheader                      Do not show the table header
  --short                            Short disply, only shows name
  -h --help      Show this screen.
  --version      Show version.

"""
import sys, operator, os, random, string
from tabulate import tabulate
from config import read_config
from connection import conn
from zone import get_zone
from docopt import docopt

__cli_name__="rbc-publicips"
from rbc_tools import __version__

def get_network_from_name(c, name):
    ret = c.list_networks(keyword=name)
    if len(ret) == 1:
        return ret[0]
    else:
        print "Unable to get network, got: %s" % ret
        sys.exit(1)

def get_id_from_address(c, address):
    try:
        res = c.list_publicipaddresses(ipaddress=address)[0]
        if res:
            return res
        else:
            print "Unable to get ip address id"
            sys.exit(0)
    except:
        print "Unable to get ip address id"
        sys.exit(1)

def print_tabulate(res, noheader=False, short=False):
    config = read_config()
    c = conn(config)
    tbl = []
    for i in res:
        if not i.__dict__.has_key('associatednetworkname'):
            if i.__dict__.has_key('vpcid'):
                i.__setattr__('associatednetworkname', "%s (VPC)" % c.list_vpcs(id=i.vpcid)[0].name)
        if i.isstaticnat:
            vmname = i.virtualmachinename
        else:
            vmname = ""
        if i.issourcenat:
            snat = "Yes"
        else:
            snat = "No"
        if short:
            tbl.append([i.name])
        else:
            tbl.append([
                i.ipaddress,
                i.zonename,
                i.associatednetworkname,
                vmname,
                snat,
                i.allocated,

                ])

    tbl = sorted(tbl, key=operator.itemgetter(2))
    if (noheader or short):
        print tabulate(tbl, tablefmt="plain")
    else:
        tbl.insert(0, ['ip', 'zone', 'network', 'staticnat', 'sourcenat', 'allocated'])
        print tabulate(tbl, headers="firstrow")


def main():
    config = read_config()
    c = conn(config)
    args = docopt(__doc__, version='%s %s' % (__cli_name__, __version__))
    if args['list']:
        res = c.list_publicipaddresses()
    elif args['allocate']:
        res = [c.associate_ipaddress(networkid=get_network_from_name(c, args['NETWORK']).id).get_result()]
    elif args['release']:
        res = [c.disassociate_ipaddress(id=get_id_from_address(c, args['IPADDRESS']).id).get_result()]
        print "Released ip address: %s" % args['IPADDRESS']
        sys.exit(0)
    if res:
        print_tabulate(res, noheader=args['--noheader'], short=args['--short'])
    else:
        print "Unable to execute command"
        sys.exit(1)



#!/usr/bin/env python
# −*− coding: UTF−8 −*−
"""Present a Ansible dynamic inventory for RBC/CloudStack

Usage:
  rbc-inventory (--host=HOST|--list)  [--refresh-cache]  [--prefix=PREFIX] [--cache_file=FILE]

Examples:
    To use the inventory, add a inventory file to your Ansible directory, the inventory should be a
    executable shell script with the following example content:
    #!/bin/bash
    RBC_SECRETKEY=xxxx
    RBC_APIKEY=xxxx
    rbc-inventory $@

Options:
  --refresh-cache                    Refresh the local inventory cache from source
  --prefix=PREFIX                    prefix all groups with this value, [default: None]
  --cache_file=FILE                  use this file for cache, [default: .rbc-inventory]
  --host=HOST                        run inventory for this host only
  --list                             run inventory for all hosts
  -h --help                          Show this screen.
  --version                          Show version.

"""
import os, sys, time, hashlib, random, base64, operator, re, json, translitcodec, time
from os import path
from config import read_config
from connection import conn
from docopt import docopt

__cli_name__ = "rbc-inventory"
from rbc_tools import __version__

# This slugify from http://flask.pocoo.org/snippets/5/
_punct_re = re.compile(r'[\t !"#$%&\'()*/<=>?@\[\\\]^_`{|},.]+')


def clear_cache(args):
    return os.remove(args['--cache_file']) if os.path.exists(args['--cache_file']) else False

def update_cache(c, args):
    try:
        i = inventory(c, args)
        with open(args['--cache_file'], 'w') as f:
            f.write(i)
    except Exception as e:
        print e
        # if we can't update cache - just use the old one
        pass
    return True


def is_cache_valid(args):
    if path.isfile(args['--cache_file']):
        mod_time = os.path.getmtime(args['--cache_file'])
        current_time = time.time()
        if (mod_time + 600) > current_time:
            return True
    return False

def load_inventory_from_cache(args):
    with open(args['--cache_file']) as f:
        cache = f.read()
    return json.dumps(json.loads(cache))


def slugify(text, delim=u'_'):
    """Generates an ASCII-only slug."""
    result = []
    for word in _punct_re.split(text.lower()):
        word = word.encode('translit/long')
        if word:
            result.append(word)
    return unicode(delim.join(result))

def prefix_key(key, prefix):
    if prefix:
        return "%s_%s" % (prefix, key)
    return key

def inventory(c, args):
    inv_args = {'state': 'Running'}
    res = c.list_virtualmachines(**inv_args)
    if args['--list']:
        _meta = dict((i.nics[0].ipaddress, {
            'rbc_hypervisor': i.hypervisor,
            'rbc_memory': i.memory,
            'rbc_cpuspeed': i.cpuspeed,
            'rbc_networkname': i.nics[0].networkname,
            'rbc_template': i.templatename,
            'rbc_account': i.account
        }) for i in res)
    # return dict
    ret = {}
    # groups
    groups = dict.fromkeys(set([ g.group for g in res if 'group' in g.__dict__ ]), None)
    for g in groups.keys():
        groups[g] = []
    for i in [ r for r in res if 'group' in r.__dict__ ]:
        groups[i.group].append(i.nics[0].ipaddress)
    ret.update(groups)
    # offerings
    offerings = dict.fromkeys(set([ i.serviceofferingname for i in res ]), None)
    for o in offerings.keys():
        offerings[o] = []
    for i in [ r for r in res]:
        offerings[i.serviceofferingname].append(i.nics[0].ipaddress)
    ret.update(offerings)
    # zones
    zones = dict.fromkeys(set([ i.zonename for i in res ]), None)
    for o in zones.keys():
        zones[o] = []
    for i in [ r for r in res]:
        zones[i.zonename].append(i.nics[0].ipaddress)
    ret.update(zones)
    # tags
    tags = dict.fromkeys(set([ ["tag_%s_%s" % (t.key, slugify(t.value)) for t in i.tags] for i in res if len(i.tags) > 0][0]), None)
    for t in tags.keys():
        tags[t] = []
    for i in [ r for r in res if len(r.tags) > 0]:
        for t in i.tags:
            tags["tag_%s_%s" % (t.key, slugify(t.value))].append(i.nics[0].ipaddress)
    ret.update(tags)
    account = { i.account: []}
    for i in [ r for r in res]:
        account[i.account].append(i.nics[0].ipaddress)
    ret.update(account)
    if args['--prefix']:
        prefix = args['--prefix']
        if prefix == "None":
            prefix = None
    # create the "prefixed" dict
    ret_prefixed = dict((prefix_key(k, prefix), v) for k, v in ret.items())
    if args['--list']:
        ret_prefixed.update({'_meta': _meta})
    return json.dumps(ret_prefixed)


def main():
    config = read_config()
    c = conn(config, method='post')
    args = docopt(__doc__, version='%s %s' % (__cli_name__, __version__))
    if args['--refresh-cache']:
        clear_cache(args)
    if args['--list']:
        try:
            if is_cache_valid(args):
                print load_inventory_from_cache(args)
            else:
                update_cache(c, args)
                print load_inventory_from_cache(args)
            #print inventory(c, args)
            sys.exit(0)
        except Exception as e:
            print "error creating inventory: %s"  % e
            sys.exit(1)
    if args['--host']:
        return {}





#!/usr/bin/env python
# −*− coding: UTF−8 −*−
import os
import molnctrl

def conn(config, method="get", timeout=120):
    rbc_secret = os.environ.get("RBC_SECRETKEY") or config.get('main', 'secretkey')
    rbc_key = os.environ.get("RBC_APIKEY")  or config.get('main', 'apikey')
    try:
        rbc_endpoint = config.get('main', 'endpoint')
    except NoOptionError:
        rbc_endpoint = os.environ.get("RBC_ENDPOINT") or "api.rbcloud.net"
    return molnctrl.Initialize(api_host=rbc_endpoint, api_key=rbc_key, api_secret=rbc_secret, req_method=method)



#!/usr/bin/env python
# −*− coding: UTF−8 −*−
"""View offerings in RedBridge Cloud

Usage:
  rbc-offerings list (compute|network) [-n] [--short]

Options:
  -n --noheader                      Do not show the table header
  --short                            Short disply, only shows name
  -h --help      Show this screen.
  --version      Show version.

"""
import sys, operator
from tabulate import tabulate
from config import read_config
from connection import conn
from zone import get_zone
from docopt import docopt

__cli_name__="rbc-offerings"
from rbc_tools import __version__
def get_offering(c, name):
    olist = []
    for o in c.list_serviceofferings(name=name):
            olist.append(o)
    if olist:
        return olist
    else:
        print "The search returned no offerings"
        sys.exit(0)

def list_main():
    config = read_config()
    c = conn(config)
    parser = ArgumentParser(description="View service offerings in RedBridge Cloud")
    parser.add_argument('-s', '--search', dest="search", action="store", help="Search for offering")
    parser.add_argument('-n', '--noheader', dest="noheader", action="store_true", default=False, help="Do not show the header")
    options = parser.parse_args()
    slist = []
    slist.append(['name', 'id', 'description'])
    if options.search:
        offerings = get_offering(c, options.search)
    else:
        offerings = c.list_serviceofferings()
    for o in offerings:
        slist.append([o.name, o.id, o.displaytext])
    print tabulate(slist, headers="firstrow")

def print_tabulate(res, offering_type="compute", noheader=False, short=False):
    config = read_config()
    c = conn(config)
    tbl = []
    for i in res:
        if short:
            tbl.append([i.name])
        else:
            if offering_type == "compute":
                tbl.append([
                    i.name,
                    i.displaytext,
                    i.storagetype,
                    ])
            elif offering_type == "network":
                tbl.append([
                    i.name,
                    i.displaytext,
                    "%s Mbit/s"% i.networkrate,
                    ])

    tbl = sorted(tbl, key=operator.itemgetter(0))
    if (noheader or short):
        print tabulate(tbl, tablefmt="plain")
    else:
        if offering_type == 'compute':
            tbl.insert(0, ['name', 'description', 'storagetype'])
        else:
            tbl.insert(0, ['name', 'description', 'external networkrate'])

        print tabulate(tbl, headers="firstrow")


def main():
    config = read_config()
    c = conn(config)
    args = docopt(__doc__, version='%s %s' % (__cli_name__, __version__))
    if args['list']:
        if args['compute']:
            res = c.list_serviceofferings()
            t = 'compute'
        elif args['network']:
            res = c.list_networkofferings(state="Enabled")
            t = 'network'
    if res:
        print_tabulate(res, noheader=args['--noheader'], short=args['--short'], offering_type=t)
    else:
        print "Unable to execute command"
        sys.exit(1)



import os, re, codecs
from setuptools import setup, find_packages
from glob import glob

name = 'rbc-tools'

with open('requirements.txt', 'r') as f:
        requires = [x.strip() for x in f if x.strip()]

conf_files = [ ('conf', glob('conf/*.cfg')) ]
dirs = [('log', [])]
data_files =  conf_files + dirs

setup(
    name=name,
    version='0.3.3',
    author='RedBridge AB',
    author_email='info@redbridge.se',
    data_files = data_files,
    url='http://github.com/redbridge/rbc-tools',
    license="Apache 2.0",
    description='Command line tools for managing RedBridge Cloud',
    long_description=open("README.rst").read(),
    packages=['rbc_tools'],
    scripts=glob('bin/*'),
    install_requires=requires,
    entry_points = {
                'console_scripts': [
                        'rbc-instances=rbc_tools.instance:main',
                        'rbc-templates=rbc_tools.template:main',
                        'rbc-networks=rbc_tools.network:main',
                        'rbc-offerings=rbc_tools.offering:main',
                        'rbc-vpns=rbc_tools.vpn:main',
                        'rbc-sshkeys=rbc_tools.sshkey:main',
                        'rbc-ipaddresses=rbc_tools.ipaddress:main',
                        'rbc-staticnat=rbc_tools.staticnat:main',
                        'rbc-portforward=rbc_tools.portforward:main',
                        'rbc-egress=rbc_tools.egress:main',
                        'rbc-setup=rbc_tools.config:main',
                        'rbc-inventory=rbc_tools.inventory:main',
                ],
    }
)



#!/usr/bin/env python
# −*− coding: UTF−8 −*−
"""Manage ssh keypairs in RedBridge Cloud

Usage:
  rbc-sshkeys list [-n] [--short]
  rbc-sshkeys generate NAME
  rbc-sshkeys delete NAME

Options:
  -n --noheader                      Do not show the table header
  --short                            Short disply, only shows name
  -h --help      Show this screen.
  --version      Show version.

"""
import sys, operator, os, random, string
import molnctrl
from tabulate import tabulate
from config import read_config
from connection import conn
from zone import get_zone
from docopt import docopt

__cli_name__="rbc-sshkeys"
from rbc_tools import __version__


def get_sshkey(c, name):
    slist = []
    for k in c.list_sshkeypairs(name=name):
            slist.append(k)
    if slist:
        return slist
    else:
        print "The search returned no ssh keys"
        sys.exit(0)

def print_tabulate(res, noheader=False, short=False):
    config = read_config()
    c = conn(config)
    tbl = []
    for i in res:
        if short:
            tbl.append([i.name])
        else:
            tbl.append([
                i.name,
                i.fingerprint,
                ])

    tbl = sorted(tbl, key=operator.itemgetter(0))
    if (noheader or short):
        print tabulate(tbl, tablefmt="plain")
    else:
        tbl.insert(0, ['name', 'fingerprint'])
        print tabulate(tbl, headers="firstrow")


def main():
    config = read_config()
    c = conn(config)
    args = docopt(__doc__, version='%s %s' % (__cli_name__, __version__))
    if args['list']:
        res = c.list_sshkeypairs()
    if args['generate']:
        try:
            res = c.create_sshkeypair(name=args['NAME'])
            print res.privatekey
            sys.exit(0)
        except molnctrl.csexceptions.ResponseError as e:
            print "Unable to create ssh key: %s" % e[1]
            sys.exit(1)
    if args['delete']:
        try:
            res = c.delete_sshkeypair(name=args['NAME'])
            print "deleted SSH key: %s" % args['NAME']
            sys.exit(0)
        except Exception as e:
            print "Unable to delete SSK keypair: %s" % e[1]
            sys.exit(1)
    if res:
        print_tabulate(res, noheader=args['--noheader'], short=args['--short'])
    else:
        print "Unable to execute command"
        sys.exit(1)



#!/usr/bin/env python
# −*− coding: UTF−8 −*−
"""Manage instances in RedBridge Cloud

Usage:
  rbc-instances list [-n] [--short ] [--search=SEARCH] [--group=GROUP] [--tags=TAGS]  [-w NETWORK]
  rbc-instances inventory [--all] [--list]
  rbc-instances get [-n] [-w NETWORK] INSTANCE
  rbc-instances destroy [-n] [-w NETWORK] INSTANCE
  rbc-instances stop [-n] [-w NETWORK] INSTANCE
  rbc-instances start [-n] [-w NETWORK] INSTANCE
  rbc-instances reboot [-n] [-w NETWORK] INSTANCE
  rbc-instances deploy [-n] [--nowait] [-i NUM] [-g GROUP] [-t TEMPLATE]
                       [-o OFFERING] [-w NETWORK] [-z ZONE] [-s SSHKEY]
                       [-d USER-DATA | -f FILE] [--tags=TAGS] INSTANCE

Examples:
  # Deploy 4 small instances with ssh key in group my-group and supply user-data
  rbc-instances deploy -i 4 -g my-group -t rbc/fedora-20-cloudimg-x86_64-10GB -o small -w default -s mykey -f /path/to/userdata  my-instance-name


Arguments:
  INSTANCE  Instance name

Options:
  -n --noheader                      Do not show the table header
  --nowait                           Do not wait for instances to start
  --short                            Short disply, only shows name
  --search=SEARCH                    Search instances matching STRING
  --tags=TAGS                        List or add tags to instances. Tags should be
                                     specified in key/value pairs: tagkey:tagvalue,app:myapp
  --all                              Include stopped instances in the inventory
  -i NUM --number=NUM                Number of instances to deploy [default: 1]
  -g GROUP --group=GROUP             Name of instance group
  -t TEMPLATE --template=TEMPLATE    Name of the template to use
                                     (use "rbc-templates list featured" to get featured templates)
  -o OFFERING --offering=OFFERING    Name of offering to use
                                     (use "rbc-offerings list" to get valid offerings)
  -w NETWORK --network=NETWORK       Name of the network to use
                                     (use "rbc-networks list" to get vaild networks)
  -z ZONE --zone=ZONE                Zone to use for deployment
                                     (use "rbc-zones list" to get vaild zones)
  -s SSHKEY --sshkey=SSHKEY          Name of SSH key to use
                                     (use "rbc-ssh-keys list" to get vaild key names)
  -d USER-DATA --user-data=USER-DATA A string with user data available for the instance
  -f FILE --user-data-file=FILE      A file containing user-data (must not exceed 2KB)
  -h --help      Show this screen.
  --version      Show version.

"""
import os, sys, time, hashlib, random, base64, operator, re
import tabulate
from argparse import ArgumentParser
from tabulate import tabulate
from config import read_config
from connection import conn
from template import get_template
from template import get_template_from_id
from offering import get_offering
from network import get_network
from zone import get_zone
from sshkey import get_sshkey
from docopt import docopt

__cli_name__="rbc-instances"
from rbc_tools import __version__

def deploy_vm(c, a):
    deploy_args = {}
    if a['--number']:
        n = int(a['--number'])
    else:
        n = 1
    if a['INSTANCE']:
        deploy_args['name'] = a['INSTANCE']
        deploy_args['displayname'] = a['INSTANCE']
    if a['--group']:
        deploy_args['group'] = a['--group']
    if a['--template']:
        try:
            template = get_template(c, a['--template'])[0]
            deploy_args['templateid'] = template.id
        except:
            print "Unable to get templateid for %s" % a['--template']
            sys.exit(1)
    if a['--offering']:
        try:
            offering = get_offering(c, a['--offering'])[0]
            deploy_args['serviceofferingid'] = offering.id
        except:
            print "Unable to get offeringid for %s" % a['--offering']
            sys.exit(1)
    if a['--network']:
        try:
            network = get_network(c, a['--network'])[0]
            deploy_args['networkids'] = network.id
        except:
            print "Unable to get networkid for %s" % a['--network']
            sys.exit(1)
    if a['--zone']:
        zone = get_zone(c, a['--zone'])[0]
        deploy_args['zoneid'] = zone.id
    else:
        zone = get_zone(c)[0]
        deploy_args['zoneid'] = zone.id
    if a['--sshkey']:
        try:
            sshkey = get_sshkey(c, a['--sshkey'])[0]
            deploy_args['keypair'] = sshkey.name
        except:
            print "Unable to get ssh key: %s" % a['--sshkey']
            sys.exit(1)
    if a['--user-data']:
        deploy_args['userdata'] = base64.b64encode(a['--user-data'])
    if a['--user-data-file']:
        if os.path.isfile(a['--user-data-file']):
            with open(a['--user-data-file']) as f:
                encoded = base64.b64encode(f.read())
            size = sys.getsizeof(encoded)
            deploy_args['userdata'] = encoded
    # Deploy it!
    res = []
    for i in range(n):
        if n > 1:
            append_hash = hashlib.new('sha1', str(random.randint(0,1000000))).hexdigest()[:3]
            deploy_args['name'] = "%s-%s" % (a['INSTANCE'], append_hash)
            deploy_args['displayname'] = "%s-%s" % (a['INSTANCE'], append_hash)
        res.append(c.deploy_virtualmachine(**deploy_args))
    vms = []
    if a['--tags']:
        tags = {}
        for tag in a['--tags'].split(','):
            regex = re.compile(r"\b(\w+)\s*:\s*([^:]*)(?=\s+\w+\s*:|$)")
            tags.update(dict(regex.findall(tag)))
    if not a['--nowait']:
        for j in res:
            if a['--tags']:
                c.create_tags(resourceids=j.id, resourcetype='UserVm', tags=tags)
            vms.append(j.get_result())
    else:
        for j in res:
            if a['--tags']:
                c.create_tags(resourceids=j.id, resourcetype='UserVm', tags=tags)
            vms.append(c.list_virtualmachines(id=j.id)[0])
    return vms

def list_vms(c, args):
    list_args = {}
    if args['--group']:
        gid = c.list_instancegroups(name=args['--group'])
        if len(gid) > 1:
            g_match = False
            for g in gid:
                if g.name == args['--group']:
                    list_args['groupid'] = g.id
                    g_match = True
            if not g_match:
                print "Multiple groups found: %s" % gid
                sys.exit(1)
        else:
            try:
                list_args['groupid'] = gid[0].id
            except IndexError:
                print "No such group found: %s" % gid
                sys.exit(1)

    if args['--search']:
        list_args['keyword'] = args['--search']

    if args['--tags']:
        tags = {}
        for tag in args['--tags'].split(','):
            regex = re.compile(r"\b(\w+)\s*:\s*([^:]*)(?=\s+\w+\s*:|$)")
            tags.update(dict(regex.findall(tag)))
        list_args['tags'] = tags
    try:
        res = c.list_virtualmachines(**list_args)
    except Exception as e:
        print "error listing instances"
        sys.exit(1)
    return res


def start_vm(c, id):
    try:
        res = c.start_virtualmachine(id=id)
    except Exception as e:
        print "error starting instance: %s"  % e
        sys.exit(1)
    return [res.get_result()]

def stop_vm(c, id):
    try:
        res = c.stop_virtualmachine(id=id)
    except Exception as e:
        print "error stopping instance: %s"  % e
        sys.exit(1)
    return [res.get_result()]

def destroy_vm(c, id):
    try:
        res = c.destroy_virtualmachine(id=id)
    except Exception as e:
        print "error destroying instance: %s"  % e
        sys.exit(1)
    return [res.get_result()]

def print_tabulate(res, noheader=False, short=False):
    config = read_config()
    c = conn(config)
    tbl = []
    template_cache = {}
    for i in res:
        if not i.__dict__.has_key('keypair'):
            i.__setattr__('keypair', 'None')
        if not i.__dict__.has_key('password'):
            i.__setattr__('password', 'None')
        if not i.__dict__.has_key('group'):
            i.__setattr__('group', 'None')
        # cache templates
        if not template_cache.has_key(i.templateid):
            # lookup
            t = get_template_from_id(c, i.templateid)
            if t:
                template_cache[i.templateid] = t
            else:
                template_cache[i.templateid] = type('Template', (object,), {'account': 'DELETED'})
        if short:
            tbl.append([i.name])
        else:
            tbl.append([i.name,
                i.zonename,
                i.state,
                i.serviceofferingname,
                i.group,
                i.nics[0].networkname,
                i.nics[0].ipaddress,
                i.keypair,
                i.password,
                "%s/%s" % (template_cache[i.templateid].account, i.templatename),
                i.created]
            )
    tbl = sorted(tbl, key=operator.itemgetter(0))
    if (noheader or short):
        print tabulate(tbl, tablefmt="plain")
    else:
        tbl.insert(0, ['name', 'zone', 'state', 'offering', 'group', 'network', 'ipaddress', 'sshkey', 'password', 'template', 'created'])
        print tabulate(tbl, headers="firstrow")

def main():
    config = read_config()
    c = conn(config, method='post')
    args = docopt(__doc__, version='%s %s' % (__cli_name__, __version__))
    res = False
    res_get = False
    if args['deploy']:
        try:
            res = deploy_vm(c, args)
        except Exception as e:
            print "error deploying instance: %s"  % e
            sys.exit(1)
    elif args['destroy']:
        try:
            id = c.list_virtualmachines(name=args['INSTANCE'])
            if len(id) == 1:
                res = destroy_vm(c, id[0].id)
            else:
                # Multiple VMs returned
                if args['--network']:
                    id = c.list_virtualmachines(name=args['INSTANCE'], networkid=get_network(c, args['--network'])[0].id)
                    res = destroy_vm(c, id[0].id)
                else:
                    print "Multiple instances with name: %s found, please supply a network name" % args['INSTANCE']
        except Exception as e:
            print "Error destroying instance: %s" % e
    elif args['stop']:
        try:
            id = c.list_virtualmachines(name=args['INSTANCE'])
            if len(id) == 1:
                res = stop_vm(c, id[0].id)
            else:
                # Multiple VMs returned
                if args['--network']:
                    id = c.list_virtualmachines(name=args['INSTANCE'], networkid=get_network(c, args['--network'])[0].id)
                    res = stop_vm(c, id[0].id)
                else:
                    print "Multiple instances with name: %s found, please supply a network name" % args['INSTANCE']
        except Exception as e:
            print "Error stopping instance: %s" % e
    elif args['start']:
        try:
            id = c.list_virtualmachines(name=args['INSTANCE'])
            if len(id) == 1:
                res = start_vm(c, id[0].id)
            else:
                # Multiple VMs returned
                if args['--network']:
                    id = c.list_virtualmachines(name=args['INSTANCE'], networkid=get_network(c, args['--network'])[0].id)
                    res = start_vm(c, id[0].id)
                else:
                    print "Multiple instances with name: %s found, please supply a network name" % args['INSTANCE']
        except Exception as e:
            print "Error starting instance: %s" % e
    elif args['list']:
        res = list_vms(c, args)
    elif args['get']:
        res = c.list_virtualmachines(name=args['INSTANCE'])
    elif args['inventory']:
        inventory(c, args)
    else:
        print "Unable to execute command"
        sys.exit(1)

    if res:
        print_tabulate(res, noheader=args['--noheader'], short=args['--short'])
    else:
        print "No virtual machines found, deploy new machines using `rbc-instances deploy`"
        sys.exit(1)

